﻿using Newtonsoft.Json;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Threading.Tasks;
using UnivateProperties_API.Containers.Timeshare;
using UnivateProperties_API.Context;
using UnivateProperties_API.Helpers;

namespace UnivateProperties_API.Repository.Timeshare
{
    public interface IResortRepository
    {
        List<ResortDisplay> GetResortsByRegion(string regionCode);
    }

    public class ResortRepository : IResortRepository
    {
        private readonly DataContext _dbContext;

        public ResortRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<ResortDisplay> GetResortsByRegion(string regionCode)
        {
            var resorts = new List<ResortDisplay>();
            List<UniPointResorts> UniResorts = new List<UniPointResorts>();
            WeekRepository weekRepo = new WeekRepository(_dbContext);
            var weeks = weekRepo.GetDtoListAll();

            var client = new RestClient(string.Format("https://www.tradeunipoint.com/unibackend/seam/resource/rest/products/resorts/{0}", regionCode))
            {
                Timeout = -1
            };
            var request = new RestRequest(Method.GET);
            IRestResponse response = client.Execute(request);
            UniResorts = JsonConvert.DeserializeObject<List<UniPointResorts>>(response.Content);

            foreach(var resort in UniResorts)
            {
                if (!string.IsNullOrEmpty(resort.ResortCode))
                {
                    ResortDisplay desp = new ResortDisplay()
                    {
                        ResortCode = resort.ResortCode,
                        ResortName = resort.ResortName
                    };
                    var imgClient = new RestClient(string.Format("https://www.tradeunipoint.com/unibackend/seam/resource/rest/products/{0}/image/1", resort.ResortCode))
                    {
                        Timeout = -1
                    };
                    var imgRequest = new RestRequest(Method.GET);
                    IRestResponse resp = imgClient.Execute(imgRequest);
                    desp.ResortImage = resp.Content;

                    if (desp.ResortImage != "")
                        desp.ResortImage = string.Format("data:image/jpeg;base64,{0}", desp.ResortImage);

                    desp.WeeksAvailable = weeks.Where(w => w.Resort.ResortCode == resort.ResortCode).Count();

                    resorts.Add(desp);
                }
            }

            return resorts.OrderBy(r => r.ResortCode).ToList();
        }
    }
}
